#include <stdio.h>
#include <stdlib.h>

#include <mpi.h>

int main (int argc, char ** argv)
{
  int n, i, size, rank;
  double d, s, x, pi;
  MPI_Status status;

  MPI_Init (&argc, &argv);
  MPI_Comm_size (MPI_COMM_WORLD, &size);
  MPI_Comm_rank (MPI_COMM_WORLD, &rank);

  if (argc != 2)
    n = 10;
  else
    n = atoi(argv[1]);

  d = 1.0/n;
  s = 0.0;

  for (i=rank+1; i<=n; i+=size) {
    x = (i-0.5)*d;
    s += 4.0/(1.0+x*x);
  }
  pi = d*s;

  if (rank != 0) {
    MPI_Send (&pi, 1, MPI_DOUBLE, 0, 1, MPI_COMM_WORLD);
  }
  else {
    double temp;

    for (i = 1; i <= size-1; i++) {
      MPI_Recv (&temp, 1, MPI_DOUBLE, i, 1, MPI_COMM_WORLD, &status);
      pi += temp;
    }
  }

  if (rank == 0) {
    fprintf ( stdout, "computed pi value = %.23f...\n", pi);
    fprintf ( stdout, "                    3.14159265358979323846264...\n");
  }

  MPI_Finalize ();
 
  return 0;
} 
